--- old/usr/src/lib/fm/topo/modules/common/fac_prov_mptsas/fac_prov_mptsas.c	2013-08-28 19:10:28.468824446 +0000
+++ new/usr/src/lib/fm/topo/modules/common/fac_prov_mptsas/fac_prov_mptsas.c	2013-08-28 19:10:28.475671914 +0000
@@ -74,12 +74,9 @@
 
 /*
  * Get or set LED state for a particular target attached to an mpt_sas
- * instance at (Enclosure Number, Slot Number).
- *
- * Returns:
- *   -2  /devices node (*devctl) does not exist
- *   -1  All other failures
- *    0  Success
+ * instance at (Enclosure Number, Slot Number).  The function returns
+ * -1 on error and sets errno to ENOENT _only_ if the /devices node
+ *  (*devctl) does not exist.
  */
 static int
 do_led_control(topo_mod_t *mod, char *devctl, uint16_t enclosure,
@@ -97,10 +94,11 @@
 	lc.LedStatus = *ledmode;
 
 	if ((fd = open(devctl, (set ? O_RDWR : O_RDONLY))) == -1) {
-		int rc = (errno == ENOENT ? -2 : -1);
+		int en = errno;
 		topo_mod_dprintf(mod, "devctl open failed: %s",
 		    strerror(errno));
-		return (rc);
+		errno = en;
+		return (-1);
 	}
 
 	if (ioctl(fd, MPTIOCTL_LED_CONTROL, &lc) == -1) {
@@ -113,9 +111,11 @@
 			 */
 			lc.LedStatus = 0;
 		} else {
+			int en = errno;
 			topo_mod_dprintf(mod, "led control ioctl failed: %s",
 			    strerror(errno));
 			(void) close(fd);
+			errno = en;
 			return (-1);
 		}
 	}
@@ -123,6 +123,7 @@
 	*ledmode = lc.LedStatus ? TOPO_LED_STATE_ON : TOPO_LED_STATE_OFF;
 
 	(void) close(fd);
+	errno = 0;
 	return (0);
 }
 
@@ -137,7 +138,7 @@
 	char *driver = NULL, *devctl = NULL;
 	uint32_t enclosure, slot;
 	uint8_t mptsas_led;
-	boolean_t set;
+	boolean_t set, done;
 	char *elem, *lastp;
 
 	if (vers > TOPO_METH_MPTSAS_LED_MODE_VERSION)
@@ -218,6 +219,7 @@
 		ret = topo_mod_seterrno(mod, EMOD_UNKNOWN);
 		goto out;
 	}
+	done = B_FALSE;
 	do {
 		topo_mod_dprintf(mod, "%s: trying mpt_sas instance at %s\n",
 		    __func__, elem);
@@ -225,20 +227,21 @@
 		ret = do_led_control(mod, elem, enclosure, slot,
 		    mptsas_led, &ledmode, set);
 
-		topo_mod_strfree(mod, elem);
-
 		/*
 		 * Only try further devctl paths from the list if this one
 		 * was not found:
 		 */
-		if (ret != -2) {
-			break;
+		if (ret == 0 || errno != ENOENT) {
+			done = B_TRUE;
 		} else {
 			topo_mod_dprintf(mod, "%s: instance not found\n",
 			    __func__);
 		}
 
-	} while ((elem = topo_mod_strsplit(mod, NULL, "|", &lastp)) != NULL);
+		topo_mod_strfree(mod, elem);
+
+	} while (!done && (elem = topo_mod_strsplit(mod, NULL, "|",
+	    &lastp)) != NULL);
 
 	if (ret != 0) {
 		topo_mod_dprintf(mod, "%s: do_led_control failed", __func__);
